package net.sergeych.crypto2

import com.ionspin.kotlin.crypto.secretbox.SecretBox
import com.ionspin.kotlin.crypto.secretbox.crypto_secretbox_NONCEBYTES
import kotlinx.serialization.Serializable

/**
 * Symmetric key implements authenticated encrypting with random nonce and optional fill.
 * Random fill is normally used when cryptanalysis of the message size is a threat.
 *
 * Do not call this constructor directly, use [randomInt] or deserialize it.
 *
 * __Algorithms:__
 *
 * - Encryption: XSalsa20 stream cipher.
 * - Authentication: Poly1305 MAC
 */
@Serializable
class SymmetricKey(
    val keyBytes: UByteArray,
) : EncryptingKey, DecryptingKey {

    /**
     * @suppress
     * nonce + ciphered data serialization aid
     */
    @Serializable
    data class WithNonce(
        val cipherData: UByteArray,
        val nonce: UByteArray,
    )

    override fun encryptWithNonce(plainData: UByteArray, nonce: UByteArray, randomFill: IntRange?): UByteArray {
        require(nonce.size == nonceByteLength)
        return SecretBox.easy(WithFill.encode(plainData, randomFill), nonce, keyBytes)
    }

    override val nonceBytesLength: Int = nonceByteLength

    override val tag by lazy { KeyTag(KeysMagickNumbers.defaultSymmetric,blake2b3l(keyBytes)) }

    override fun decryptWithNonce(cipherData: UByteArray, nonce: UByteArray): UByteArray =
        protectDecryption {
            WithFill.decode(SecretBox.openEasy(cipherData, nonce, keyBytes))
        }

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other !is SymmetricKey) return false

        return keyBytes contentEquals other.keyBytes
    }

    override fun hashCode(): Int {
        return keyBytes.contentHashCode()
    }


    companion object {
        /**
         * Create a secure random symmetric key.
         */
        fun random() = SymmetricKey(SecretBox.keygen())

        val nonceByteLength = crypto_secretbox_NONCEBYTES
    }

}